from configparser import ConfigParser
from zsub.common.web import site_check
from zsub.common.search import search
from zsub.common.reduce import reduce
from zsub.common.extract import extract
from zsub.common.language import Language
from zsub.commands.dl import download, get_dlinks


def batch(req):
    if req.add:
        site = site_check(req.site)
        add_title(req.config, req.site, req.captcha)
    elif req.print:
        print_lists(req.config)
    elif req.reset:
        fresh_lists(req.config)
    else:
        site = site_check(req.site)
        to_extract = batch_download(req.config, site, req.cache)
        extract(to_extract, req.unzip, req.group, req.open_after)


def add_title(path, site, captcha):
    db = check_batchini(path)
    ini = ConfigParser()
    ini.read(db)
    print_lists(path)
    go_on = True
    while go_on:
        section = get_section(ini)
        titles = get_title(site, captcha)
        for title in titles:
            langs = input(f'interested language(s) for {title} [en]: ') or 'en'
            section[title] = langs
        with open(db, 'w+') as ini_file:
            print(f'\n[add] "{title}" is added to "{ section}" list\n')
            ini.write(ini_file)
        go_on = keep_go_on()

def batch_download(path, site, cache, to_download=[]):
    db = check_batchini(path)
    print_lists(path)
    ini = ConfigParser()
    ini.read(db)
    section = get_section(ini)
    for key in section:
        _path = '/subtitles/' + key
        print(f'\n[check] {key}')
        languages = Language(*section[key].split(" "))
        to_download = reduce([_path,], site, languages.abbrs, cache, to_download, early_exit=False)
    if not to_download:
        exit()
    to_download = get_dlinks(to_download, site, cache)
    to_extract = download(to_download)
    if not to_extract:
        exit()
    return to_extract

def get_section(ini):
    sections = ini.sections()
    print('\nwhich list? ', flush=True, end='')
    while True:
        print('{' + ', '.join(sections)+ '}')
        section = input(f'[{sections[0]}]: ') or sections[0]
        if section.lower() in sections:
            break
        else:
            print('not an option. try again')         
    return ini[section.lower()]

def get_title(site, captcha):
    print()
    while True:
        title = input('title to search: ')
        paths = search(title, site, captcha, early_exit=False)
        if not paths:
            continue
        else:
            return [path.split('/')[-1] for path in paths]

def keep_go_on():
    add_more = input('\nadd more? [Y/n]: ') or 'y'
    if add_more.lower() == 'n':
        return False
    else:
        print('*******************************************')
        return True

def fresh_lists(path):
    path.parent.mkdir(parents=True, exist_ok=True)
    db = path.joinpath('batch.ini')
    sections = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday', 'hot']
    lists = ConfigParser()
    for section in sections:
        lists.add_section(section)
    with open(db, 'w+') as file:
        lists.write(file)

def print_lists(path):
    db = check_batchini(path)
    with open(db, 'r') as file:
        text = file.read()
        lines = [line for line in text.splitlines() if line]
        print('')
        print('-------batch.ini------------')
        print('\n'.join(lines))
        print('----------------------------')

def check_batchini(path):
    path.parent.mkdir(parents=True, exist_ok=True)
    db = path.joinpath('batch.ini')
    if not db.exists():
        fresh_lists(path)
    return db
