from zipfile import ZipFile
from requests_futures.sessions import FuturesSession
from zsub.common.web import get_dl_btn, site_check
from zsub.common.search import search
from zsub.common.reduce import reduce
from zsub.common.extract import extract


def dl(req):
    site = site_check(req.site)
    paths = search(req.title, site, req.captcha, req.auto_select)
    links = reduce(paths, site, req.languages.abbrs, req.cache)
    if req.simulation:
        simulate_download(links, site, req.cache)
    else:
        to_download = get_dlinks(links, site, req.cache)
        to_extract = download(to_download)
        extract(to_extract, req.unzip, req.group, req.open_after)

def get_dlinks(links, site, cache, to_download=[]):
    print('[Get Links]')
    session = FuturesSession(max_workers=8)
    futures = []
    n = len(links)
    for link in links:
        futures.append(session.get(site.base_url + link))
    for i, path in enumerate(links):
        file = cache.joinpath('.' + path + '.zip')
        url = site.base_url + path
        print(f'  [{i+1}/{n}] {url}')
        page = futures[i].result()
        link = get_dl_btn(page, site.selectors['btn'], early_exit=False)
        if not(link):
            print('      no download link')
            continue
        to_download.append({'path': file, 'url': url, 'dlink': site.base_url+link['href']})

    return to_download

def download(to_download, to_extract=[]):
    print("[Download]")
    n = len(to_download)
    session = FuturesSession(max_workers=8)
    for sub in to_download:
        sub['future'] = session.get(sub['dlink'])
    for i, sub in enumerate(to_download):
        file = sub['path']
        file.parent.mkdir(parents=True, exist_ok=True)
        print(f'  [{i+1}/{n}] {sub["path"]}')
        with open(file, "w+b") as f:
            file_content = sub['future'].result().content
            f.write(file_content)
        to_extract.append(sub)

    return to_extract

def simulate_download(links, site, cache):
    print("[Simulate]")
    n = len(links)
    for i, path in enumerate(links):
        file = cache.joinpath('.' + path + '.zip')
        file.parent.mkdir(parents=True, exist_ok=True)
        url = site.base_url + path
        print(f'  [{i+1}/{n}] {url}')
        with ZipFile(file, 'w'):
            pass
