from zsub.common.utils import get_size, isempty, get_title, get_sorted_items
from zsub.common.utils import count_children, count_grandchildren, count_total
from zsub.common.version import __version__

LANGMAX = 15
SIZEMAX = 9
COUNTMAX = 6
SPACING = 3
BAR = 41
SYMBOL = '#'


def info(req):
    path = req.cache.joinpath("subtitles")
    path.mkdir(parents=True, exist_ok=True)
    if isempty(path):
        print('\n' + "[Exit] Cache folder is empty." + '\n')
    else:
        size = get_size(path)
        children = count_children(path)
        total = count_total(path)
        basic_info(path, size, children, total)
        print()
        if req.verbosity == 1:
            level_one(path, req.sort)
        elif req.verbosity >= 2:
            level_two(path)
        print()
    exit()


def basic_info(path, size, children, total):
    print(f"\nzsub ver {__version__}")
    print('\n[basic info]')
    print(' '*2 + f'Path:       {path}')
    print(' '*2 + f'Size:       {size}')
    print(' '*2 + f'Titles:     {children}')
    print(' '*2 + f'Subtitles:  {total}')


def level_one(path, sort_key):
    items = get_sorted_items(path, sort_key)
    tick = {'s': '', 'n': '', 't': ''}
    tick[sort_key] = SYMBOL
    header = f'{tick["n"]}Files'.rjust(COUNTMAX) + ' '*SPACING +\
        f'{tick["s"]}Size'.rjust(SIZEMAX) + ' '*SPACING +\
        f'{tick["t"]}Title'
    subheader = "="*COUNTMAX + " "*SPACING + "="*SIZEMAX + " "*SPACING + "="*BAR
    print(header)
    print(subheader)
    for item in items:
        print(str(item['n']).rjust(COUNTMAX), end=' '*SPACING)
        print(item['s'].rjust(SIZEMAX), end=' '*SPACING)
        print(item['t'], flush=True)
    print(subheader)
    print(header)
    print(f'\nnotice: results are sorted by {SYMBOL} marked column\n')

def level_two(path):
    for child in path.iterdir():
        print('\n' + ' '*8 +  get_title(child))
        print('-'*BAR)
        for lang in child.iterdir():
            print(f'{lang.name}:'.ljust(LANGMAX), end='')
            print(str(count_children(lang)).rjust(
                COUNTMAX) + ' file(s) [', end='')
            print(get_size(lang).rjust(SIZEMAX)+']')
        print(' '*(LANGMAX-6) + '-'*(BAR-LANGMAX+6))  # 6 is length of word 'total:'
        print('total:'.rjust(LANGMAX), end='')
        print(str(count_grandchildren(child)).rjust(
            COUNTMAX) + ' file(s) [', end='')
        print(get_size(child).rjust(SIZEMAX)+']')
        print()
