from zsub.common.web import get_elements
from zsub.common.language import abbr2full


def reduce(paths, site, langs, cache, to_download=[], early_exit=True):
    for path in paths:
        results = language_filter(path, site, langs, early_exit)
        if may_exit(results):
            continue
        results = unique_filter(results)
        results = downloaded_filter(results, cache)
        if may_exit(results):
            print('[warning] nothing to download.\n')
            continue
        to_download = to_download[:] + results[:]
        print(f'\n## total: {len(to_download)} subtitles to download\n')
    may_exit(to_download, early_exit)
    return to_download

def may_exit(to_download, early_exit=False):
    if not to_download:
        if early_exit:
            print('\n[Warning] nothing to download.\n[Exit]')
            exit()
        return True
    else:
        return False


def language_filter(path, site, langs, early_exit):
    results = list()
    url = site.base_url + path
    links = get_elements(url, site.selectors['link'], early_exit)
    print(url)
    print('[Filters]')
    print('  1) Languages:')
    for lang in langs:
        language = abbr2full(lang)
        has_lang = [link for link in links if link['href'].split('/')[-2] == language]
        print(' '*6 + f'{language}({lang}): {len(has_lang)}')
        results = results[:] + has_lang[:]
    return results

def unique_filter(results):
    uniques = {link.attrs['href'] for link in results}
    print(f'  2) remove duplicates: {len(results)} -> {len(uniques)}')
    return uniques

def downloaded_filter(results, cache):
    new_paths = [path for path in results if not cache.joinpath('.' + path + '.zip').exists()]
    print(f'  3) remove downloaded: {len(results)} -> {len(new_paths)}')
    return new_paths