import sys
from pathlib import Path
from zsub.mirrors import get_mirror
from zsub.common.language import Language
from zsub.common.arguments import get_parser
from zsub.common.configs import Configs, get_configs

def get_title():
    print('\n[title]')
    while True:
        title = input(' '*4 + 'title to search: ') or ''
        if title:
            print()
            return title.split()

class Request(Configs):
    def __init__(self):
        super().__init__()
        configs = get_configs(self.config.joinpath('.', 'config.ini'))
        user = configs['User']
        self.open_after = user.getboolean('open_after')
        self.auto_select = user.getboolean('auto_select')
        self.site = get_mirror(user.get('site'))
        langs = user.get('languages').split()
        self.languages = Language(*langs)
        self.unzip = Path(user.get('unzip_folder'))
        
        parser = get_parser()
        args = parser.parse_args(sys.argv[1:])

        self.command = args.command

        if args.__contains__('title'):
            if not(args.title):
                args.title = get_title()
            self.title = " ".join(args.title)
        
        if args.__contains__('languages'):
            if args.languages:
                self.languages = Language(*args.languages)

        if args.__contains__('destination'):
            if args.destination:
                self.unzip = Path(args.destination)

        if args.__contains__('site'):
            if args.site:
                self.site = get_mirror(args.site)

        if args.__contains__('reset'):
            self.reset = args.reset

        if args.__contains__('add'):
            self.add = args.add

        if args.__contains__('print'):
            self.print = args.print

        if args.__contains__('zero'):
            self.zero = args.zero

        if args.__contains__('simulation'):
            self.simulation = args.simulation

        if args.__contains__('verbosity'):
            self.verbosity = args.verbosity or 0

        if args.__contains__('number_sort'):
            if args.number_sort:
                self.sort = 'n'
            elif args.size_sort:
                self.sort = 's'
            else:
                self.sort = 't'

        if args.__contains__('no_group'):
            if self.command in ['dl', 'unzip']:
                self.group = args.group and args.no_group
            elif self.command == 'batch':
                self.group = not args.no_group

        if args.__contains__('auto_select'):
            if '-A' in str(sys.argv[1:]):
                self.auto_select = args.auto_select
            else:
                self.auto_select = args.auto_select or self.auto_select
        
        if args.__contains__('open_after'):
            if '-O' in str(sys.argv[1:]):
                self.open_after = args.open_after
            else:
                self.open_after = args.open_after or self.open_after
