
from zsub.common.web import get_search_results
from zsub.common.select import select
from zsub.common.utils import get_close_matches


def search(title, site, captcha, auto_select=False, early_exit=True):
    titles = get_search_results(site, title, captcha, early_exit)
    aggregated = {title.attrs['href']: title.text for title in titles}
    results = [{'path': path, 'title': title}
               for path, title in aggregated.items()]
    may_exit(results, early_exit)
    selected = select(results, auto_select)
    return get_paths(selected, results, site)


def cache_search(title, languages, cache, auto_select=False):
    path = cache.joinpath("subtitles")
    path.mkdir(parents=True, exist_ok=True)
    results = get_close_matches(path, title)
    may_exit(results, early_exit=True)
    selected = select(results, auto_select)
    paths = get_paths(selected, results)
    return get_path_lang(paths, languages)

def get_paths(selected, results, site=''):
    base = site.base_url if site else ''
    paths = list()
    for s in selected:
        s -= 1
        print(' '*6 + f'"{results[s]["title"]}"')
        print(' '*6 + f'"{base}{results[s]["path"]}"\n')
        paths.append(results[s]['path'])
    return paths


def get_path_lang(paths, languages):
    final_paths = list()
    for path in paths:
        for language in languages:
            p = path.joinpath(language)
            if p.exists():
                final_paths.append(p)
            else:
                print(' '*4 + f"no result for: {language} in {path.absolute()}")
    return final_paths


def may_exit(results, early_exit):
    if not results:
        print('\n' + '[Warning] No result for this title.' + '\n')
        if early_exit:
            print('[Exit]')
            exit()
