import requests
from bs4 import BeautifulSoup
from zsub.mirrors import MIRRORS, get_mirror


SIGNS = [
    "Temporary unavailable",
    "Request Timeout",
    "please retry a few minutes later.",
    'Backend server error',
    'many requests'
]

BAD = [
    "Bad request"
]


def available(pagetext):
    for sign in SIGNS:
        if sign.lower() in str(pagetext).lower():
            print('\n[Warning] Temporary unavailable. Try again later or check the site in browser.')
            return False
    for sign in BAD:
        if sign.lower() in str(pagetext).lower():
            print('\n[Warning] Bad request. maybe captcha is expired. login again with "zsub login"\n')
            return False
    return True

def could_exit(text, early_exit):
    if not available(text):
        if early_exit:
            print('[Exit]')
            exit()

def get_elements(url, selector, early_exit=True):
    '''early exit must be false in batch mode'''
    page = requests.get(url)
    page.encoding = 'utf-8'
    elements = BeautifulSoup(page.text, 'lxml').select(selector)
    could_exit(page.text, early_exit)
    return elements

def get_dl_btn(page, selector, early_exit=True):
    '''early exit must be false in batch mode'''
    page.encoding = 'utf-8'
    btn = BeautifulSoup(page.text, 'lxml').select_one(selector)
    could_exit(page.text, early_exit)
    return btn

def get_search_results(site, title, captcha, early_exit=True):
    url = site.base_url + site.query_path
    if site.method == 'get':
        url += f"?query={title}&l=''"
        print(f'[Search] {url}')
        page = requests.get(url)
    elif site.method == 'post':
        data = {'query': title, 'l':'', 'g-recaptcha-response': captcha}
        print(f'[Search] {url}')
        page = requests.post(url, data=data)
    page.encoding = 'utf-8'
    elements = BeautifulSoup(page.text, 'lxml').select(site.selectors['title'])
    could_exit(page.text, early_exit)
    return elements

def is_responding(site):
    try:
        print(f"[{site.name}] {site.base_url} is ", flush=True, end='')
        status = requests.get(site.base_url).status_code
        if status == 200:
            print('OK')
            return True
        else:
            print(f'down? (status: {status})')
            return False
    except requests.exceptions.ConnectionError:
        print('down? (Connection Error)')
        return False
    except KeyboardInterrupt:
        print('\n'*2 + '[Exit] Terminated by user.' + '\n')
        exit()
    except Exception as e:
        print('down?')
        print(e)
        return False

def site_check(site):
    if is_responding(site):
        return site
    # if requested site is not responding, first responding site will be selected.
    names = MIRRORS[:]
    names.remove(site.name)
    for name in names:
        site = get_mirror(name)
        if is_responding(site):
            return site
    else:
        print('\n[Warning] All sites are not responding. Please check your internet connection.\n')
        print('Exit')
        exit()


def get_token(url, username='fakeuser', password='verystrong'):
    session = requests.Session()
    session.get(url)
    return session.cookies.get_dict()['idsrv.xsrf']