import uuid
import zipfile
import rarfile
from zsub.common.language import get_lang
from zsub.common.utils import open_folder, remove_same


class BadCompressedFile(Exception):
    pass


def extract(to_extract, destination, group, open_after=False):
    print('[Extract]')
    to_remove = set()
    for i, item in enumerate(to_extract):
        file = item['path']
        dest = make_destination(file, destination, group)
        try:
            _extract(file, i, dest)
            to_remove.add(dest)
        except BadCompressedFile:
            print(' '*4 + '...file is not zip or rar. delete...')
            if item['url']:
                print(' '*4 + f"...downloaded from: {item['url']}")
            file.unlink()
        except FileNotFoundError:
            print('[error] file is not found. maybe unrar is not installed.')
        except rarfile.RarCannotExec:
            print('\n\n**** unrar is not installed.****')
            print(
                '**** visit https://github.com/7aman/zsub/wiki/Install#3---unrar for more information.****\n\n')
    remove_same(to_remove)
    open_folder(destination, open_after)


def get_extracor(file):
    path = str(file.absolute())
    if zipfile.is_zipfile(file):
        return zipfile.ZipFile
    elif rarfile.is_rarfile(path):
        return rarfile.RarFile
    else:
        raise BadCompressedFile


def make_destination(file, destination, group):
    dest = destination
    if group:
        parents = str(file).split('/')
        dest = destination.joinpath(*parents[-3:-1])
    dest.mkdir(parents=True, exist_ok=True)
    return dest


def _extract(file, i, dest):
    print(f' [{i+1}] {file}')
    language = get_lang(file)
    Extractor = get_extracor(file)
    with Extractor(str(file)) as compressed:
        for item in compressed.namelist():
            print(' '*4 + item)
            path_org = dest.joinpath(item)
            path_lng = path_org.with_suffix('.' + language + path_org.suffix)
            if path_lng.exists():
                rand_str = str(uuid.uuid4())[0:3]
                new_name = path_lng.with_suffix('.' + rand_str + path_lng.suffix)
                path_lng.rename(new_name)
            compressed.extract(item, str(dest))
            path_org.rename(path_lng)